package com.eco.plugin.xxx.mkssso.filter;

import com.eco.plugin.xxx.mkssso.config.PluginSimpleConfig;
import com.eco.plugin.xxx.mkssso.utils.EncryptUtils;
import com.eco.plugin.xxx.mkssso.utils.FRUtils;
import com.eco.plugin.xxx.mkssso.utils.Utils;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.json.JSONObject;
import com.fr.plugin.context.PluginContexts;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.fun.Authorize;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

@EnableMetrics
@Authorize(callSignKey = "com.eco.plugin.xxx.mkssso")
public class SSOFilter extends AbstractGlobalRequestFilterProvider {
    @Override
    public String filterName() {
        return "mksssoFilter";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{"/*"};
    }

    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain chain ){

        if(PluginContexts.currentContext().isAvailable()){
            String url2 = FRUtils.getAllUrl(req);

            //登出接口
            if(url2.contains("cross/domain")){
                HttpSession session = req.getSession(true);
                session.invalidate();
                release(req,res,chain);
                return ;
            }

            String code = req.getParameter("access_token");

            //是否放行
            boolean release = isRelease(req);

            if(release){
                release(req,res,chain);
                return;
            }

            PluginSimpleConfig psc = PluginSimpleConfig.getInstance();

            if(Utils.isNullStr(code)){
                //跳转认证中心
                redirect(req,res,psc);
                return;
            }

            //获取userInfo
            String username = getUsername(code);

            //登录
            String idtoken = req.getParameter("id_token");
            HttpSession session = req.getSession(true);
            String url = Utils.getRedirectUrl(req,"id_token");
            session.setAttribute("idtoken",idtoken);
            FRUtils.login(req,res,username,url);

        }

        release(req,res,chain);
    }

    private static Map<String,String> getParamMap(String url){
        Map<String,String> result = new HashMap<String,String>();

        if(url.contains("#")){
            int index = url.lastIndexOf("#");
            String querystr = url.substring(index,url.length()-1);
            String[] params =querystr.split("&");

            for(int i=0;i<params.length;i++){
                String[] param = params[i].split("=");
                String key = param[0];
                String value = param[1];

                result.put(key,value);
            }
        }

        return result;
    }

    private boolean isRelease(HttpServletRequest req) {
        String url = FRUtils.getAllUrl(req);
        FRUtils.FRLogInfo("requestUrl:"+url);
        String reft = req.getParameter("ref_t");

        boolean isLogin = FRUtils.isLogin(req);
        boolean isRemote = url.contains("remote");
        boolean isLoginPage = url.contains("login")||url.contains("decision/file")||url.contains("decision/resource")||url.contains("decision/system")||url.contains("query/ip");
        boolean isViewlt = url.contains("viewlet") || "design".equals(reft) || url.contains("view/report") || url.contains("view/form");
        boolean isMobiel = Utils.isMobile(req);
        return isLogin || isRemote || isLoginPage || isViewlt || url.contains("webroot/sso") || isMobiel || url.contains("logout");
    }

    //跳转认证中心
    private void redirect(HttpServletRequest req,HttpServletResponse res, PluginSimpleConfig psc) {
        String redirect = psc.getIndex();
        FRUtils.FRLogInfo("index:"+redirect);
        redirect = redirect.replace("/decision","");

        redirect += "/sso/appLogin.html";
        FRUtils.FRLogInfo("index2:"+redirect);

        String authurl = psc.getAuthUrl();
        authurl += "?client_id="+psc.getClientId()+"&scope="+psc.getSecret()+"&response_type=token+id_token&redirect_uri="+redirect+"&state=abc&nonce=xyz&grant_type=implicit";
        try {
            FRUtils.FRLogInfo("sso authurl ->"+authurl);
            res.sendRedirect(authurl);
        } catch (IOException e) {
            FRUtils.FRLogInfo("跳转认证中心异常:"+e.getMessage());
        }
    }

    //获取用户名
    private String getUsername(String token) {
        String[] tokens =token.split("\\.");
        JSONObject userinfo = new JSONObject(EncryptUtils.base64Decode(tokens[1]));
        String username = userinfo.getString("name");
        return username;
    }

    //放行拦截器
    private void release(HttpServletRequest req, HttpServletResponse res, FilterChain chain) {
        try{
            chain.doFilter(req,res);
        }catch (Exception e){
            FRUtils.FRLogInfo("拦截失败");
        }
    }
}

