package com.eco.plugin.xxx.mkssso.utils;

import com.fr.log.FineLoggerFactory;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.DESKeySpec;
import java.io.IOException;
import java.security.MessageDigest;
import java.security.SecureRandom;

public class EncryptUtils {

    /**
     * sha 加密
     * @param str
     * @return
     */
    public static String sha(String str){
        String sha256Str = "";

        try {
            MessageDigest sha256Deget = MessageDigest.getInstance("SHA-256");
            byte[] sha256Encode = sha256Deget.digest(str.getBytes());
            sha256Str = ByteToHexStr(sha256Encode);
        }catch (Exception e){
            FineLoggerFactory.getLogger().info("FRLOG:SHA256加密异常:"+e.getMessage());
        }

        return sha256Str;
    }

    /**
     * byte数组转16进制字符串
     * @param bytes
     * @return
     */
    private static String ByteToHexStr(byte[] bytes)
    {
        String hexStr = "";

        for(int i =0;i<bytes.length;i++)
        {
            int temp = bytes[i] & 0xff;
            String tempHex = Integer.toHexString(temp);
            if(tempHex.length() < 2)
            {
                hexStr += "0"+tempHex;
            }
            else {
                hexStr += tempHex;
            }
        }

        return hexStr;
    }

    /**
     * aes 加密
     * @param str
     * @param privateKey
     * @return
     */
    public static String aesEncrypt(String str,String privateKey){
        try {
            // 生成密钥对象
            SecretKey secKey = generateAesKey(privateKey.getBytes());

            // 获取 AES 密码器
            Cipher cipher = Cipher.getInstance("AES");
            // 初始化密码器（加密模型）
            cipher.init(Cipher.ENCRYPT_MODE, secKey);

            // 加密数据, 返回密文
            byte[] cipherBytes = cipher.doFinal(str.getBytes());
            return new BASE64Encoder().encodeBuffer(cipherBytes);
        } catch (Throwable e) {
            FRUtils.FRLogInfo("aes 加密异常 " + e.getMessage());
        }

        return null;
    }

    /**
     * 生成密钥对象
     */
    private static SecretKey generateAesKey(byte[] key) throws Exception {
        // 创建安全随机数生成器
        SecureRandom random = SecureRandom.getInstance("SHA1PRNG");
        // 设置 密钥key的字节数组 作为安全随机数生成器的种子
        random.setSeed(key);

        // 创建 AES算法生成器
        KeyGenerator gen = KeyGenerator.getInstance("AES");
        // 初始化算法生成器
        gen.init(128, random);

        // 生成 AES密钥对象, 也可以直接创建密钥对象: return new SecretKeySpec(key, ALGORITHM);
        return gen.generateKey();
    }

    /**
     * aes 解密
     * @param str
     * @param privateKey
     * @return
     */
    public static String aesDecrypt(String str,String privateKey){
        try {
            // 生成密钥对象
            SecretKey secKey = generateAesKey(privateKey.getBytes());

            // 获取 AES 密码器
            Cipher cipher = Cipher.getInstance("AES");
            // 初始化密码器（加密模型）
            cipher.init(Cipher.DECRYPT_MODE, secKey);

            // 加密数据, 返回密文
            byte[] cipherBytes = cipher.doFinal(new BASE64Decoder().decodeBuffer(str));
            return new String(cipherBytes);
        } catch (Throwable e) {
            FRUtils.FRLogInfo("aes 解密异常 " + e.getMessage());
        }

        return null;
    }

    /**
     * base64加密
     * @param key
     * @return
     */
    public static String base64Encode(String key){
        return (new BASE64Encoder()).encodeBuffer(key.getBytes());
    }

    /**
     * base64加密
     * @param key
     * @return
     */
    public static String base64Encode(byte[] key){
        return (new BASE64Encoder()).encodeBuffer(key);
    }

    /**
     * base64解密
     * @param key
     * @return
     */
    public static String base64Decode(String key){
        String result = "";
        try {
            result = new String((new BASE64Decoder()).decodeBuffer(key));
        } catch (IOException e) {
            FineLoggerFactory.getLogger().info("FRLOG:BASE64解密异常:"+e.getMessage());
        }

        return result;
    }

    /**
     * base64解密
     * @param key
     * @return
     */
    public static byte[] base64DecodeB(String key){
        byte[] result = null;
        try {
            result = (new BASE64Decoder()).decodeBuffer(key);
        } catch (IOException e) {
            FineLoggerFactory.getLogger().info("FRLOG:BASE64解密异常:"+e.getMessage());
        }

        return result;
    }

    /**
     * 是否被base64加密过
     * @param str
     * @return
     */
    public static boolean isBase64(String str) {
        if (str == null || str.trim().length() == 0) {
            return false;
        }
        else {
            if (str.length() % 4 != 0) {
                return false;
            }

            char[] strChars = str.toCharArray();

            for (char c:strChars) {
                if ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') || (c >= '0' && c <= '9')
                        || c == '+' || c == '/' || c == '=') {
                    continue;
                }
                else {
                    return false;
                }
            }

            return true;
        }
    }

    /**
     * des加密
     * @param datasource
     * @param password
     * @return
     */
    public static String desEncrypt(String datasource, String password) {
        try {
            SecureRandom random = new SecureRandom();
            DESKeySpec desKey = new DESKeySpec(password.getBytes());
            // 创建一个密匙工厂，然后用它把DESKeySpec转换成
            SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
            SecretKey securekey = keyFactory.generateSecret(desKey);
            // Cipher对象实际完成加密操作
            Cipher cipher = Cipher.getInstance("DES");
            // 用密匙初始化Cipher对象
            cipher.init(Cipher.ENCRYPT_MODE, securekey, random);
            // 现在，获取数据并加密
            // 正式执行加密操作
            return base64Encode(cipher.doFinal(datasource.getBytes()));
        } catch (Throwable e) {
            FRUtils.FRLogInfo("des 加密异常 " + e.getMessage());
        }

        return null;
    }

    /**
     * des 解密
     * @param src
     * @param password
     * @return
     * @throws Exception
     */
    public static String desDecrypt(String src, String password) throws Exception {
        // DES算法要求有一个可信任的随机数源
        SecureRandom random = new SecureRandom();
        // 创建一个DESKeySpec对象
        DESKeySpec desKey = new DESKeySpec(password.getBytes("UTF-8"));
        // 创建一个密匙工厂
        SecretKeyFactory keyFactory = SecretKeyFactory.getInstance("DES");
        // 将DESKeySpec对象转换成SecretKey对象
        SecretKey securekey = keyFactory.generateSecret(desKey);
        // Cipher对象实际完成解密操作
        Cipher cipher = Cipher.getInstance("DES");
        // 用密匙初始化Cipher对象
        cipher.init(Cipher.DECRYPT_MODE, securekey, random);
        // 真正开始解密操作
        return new String(cipher.doFinal(base64DecodeB(src)));
    }
}
